package chap10.program03

import scala.util.parsing.input.CharSequenceReader

case class Token(val tokenType: String, val value: String) {
  override def toString = "[" + tokenType + ": " + value + "]"
}

class SimpleLexer(input: String) {
  val OPERATOR = "OPERATOR"
  val NUMBER = "NUMBER"
  val EOF = "EOF"
  var reader = new CharSequenceReader(input)

  def nextToken() : Token = { // 文字列からトークンを切り出して返す
    while(!reader.atEnd) {
      reader.first match {
        case c:Char if (c == '+' || c == '-')
          => return new Token(OPERATOR, takeFirst)
        case c:Char if (c >= '0' && c <= '9')
          => return new Token(NUMBER, takeFirst)
        case c:Char if (c == ' ' || c == '\t' || c == '\r' ||
                        c == '\n')
          => dump // 空白は除去（スキップ）
        case unknown =>
          throw new Exception("Unknown Token(" + unknown + ")")
      }
    }
    new Token(EOF, "") // 文字列の終端
  }

  def takeFirst() = { // 文字シーケンスから先頭の文字を削除して返す
    val first = reader.first
    reader = reader.drop(1)
    first.toString
  }

  def dump() : Unit = reader.first match { // 空白は除去
    case c:Char if (c == ' ' || c == '\t' || c == '\r' || c == '\n')
      => takeFirst; dump
    case _ =>
  }
}
