package chap11.program01

import java.util.Date
import java.text.SimpleDateFormat

object MainApplication {
  // 以下セマンティックモデル
  class Time(val hour:Int, val minute:Int) {
    override def toString =
      hour + ":" + "%02d".format(minute)
  }

  class TimeRange(val startTime:Time ,val endTime:Time) {
    override def toString =
      "(from " + startTime + " to " + endTime + ")"
  }

  class Place(val name:String) {
    override def toString = "@" + name
  }

  class Schedule(val content:String,
                 val timeRange:TimeRange,
                 val place:Option[Place] ) {
    override def toString = (content +
                            (place.getOrElse("")) +
                             timeRange)
  }

  // 以下エクスプレッションビルダー
  class TimeBuilder(val hour:Int) {
    def :/(minute:Int)= new Time(hour,minute)
  }

  class ScheduleWithPlaceBuilder(val content:String,
                                 val place:Option[Place]) {
    def from (startTime:Time) =
      new ScheduleStartTimeBuilder(content, startTime, place)
  }

  class contentBuilder(content:String) {
    def at(placeString:String) =
      new ScheduleWithPlaceBuilder(content,
                                   Some(new Place(placeString)))
    def from (startTime:Time) =
      new ScheduleStartTimeBuilder(content, startTime, None)
  }

  class ScheduleStartTimeBuilder(content:String,
                                 startTime:Time,
                                 place:Option[Place]) {
    def to(endTime:Time) =
      new Schedule(content,
                   new TimeRange(startTime, endTime),
                   place)
  }

  implicit def Int2TimeBuilder(hour:Int) = new TimeBuilder(hour)

  implicit def
    String2ContentBuilder(content:String) =
      new contentBuilder(content)

  def main(args:Array[String]){
    val time = 12:/00
    val scheduleWithPlace =
      "Scalaの勉強" at "図書館" from 10:/30 to 12:/30
    val scheduleWithoutPlace = "食事" from 12:/30 to 13:/30

    println(time)
    println(scheduleWithPlace)
    println(scheduleWithoutPlace)
  }
}
